package be.iminds.ilabt.jfed.espec.model;

import javax.annotation.Nonnull;
import java.util.Objects;

/**
 * FileSource
 */
public class FileSource {

    public enum SourceType {
        /**
         * The file is in the "bundle" (archive, dir, ..) in which the espec is
         */
        BUNDLED,
        /**
         * The file is specified inside the espec
         */
        DIRECT,
        /**
         * The file is in a public github repo
         */
        GITHUB,
        /**
         * The file is in a public git repo
         */
        GIT,
        /**
         * The file should be downloaded (URL)
         */
        DOWNLOAD,
        /**
         * The file should be generated by jFed
         */
        GENERATED,
        /**
         * The file should contain meta-data about the experiment/slice
         */
        META

        /*
          Note that there is another option: NO filesource   (= file already present)
         */
    }
    @Nonnull protected final SourceType type;
    @Nonnull protected final String value;
    @Nonnull protected final String basename;

    public FileSource(@Nonnull SourceType type, @Nonnull String value, @Nonnull String basename) {
        this.type = type;
        this.value = value;
        this.basename = basename;
    }

    @Nonnull
    public SourceType getType() {
        return type;
    }

    @Nonnull
    public String getValue() {
        return value;
    }

    @Nonnull
    public String getBasename() {
        return basename;
    }

    /**
     * conveniance method that returns true for type GENERATE and value KEYPAIR*
     * @return
     */
    public boolean isGeneratedKeyPair() {
        return type == FileSource.SourceType.GENERATED &&
                value.trim().toLowerCase().startsWith("keypair");
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        FileSource that = (FileSource) o;
        return type == that.type &&
                Objects.equals(value, that.value) &&
                Objects.equals(basename, that.basename);
    }

    @Override
    public int hashCode() {
        return Objects.hash(type, value, basename);
    }

    @Override
    public String toString() {
        return "FileSource(" +
                "" + type + "" +
                " \"" + value + "\"" +
                " (\"" + basename + "\")" +
                ')';
    }
}
